@@ -1,18 +1,19 @@
 use super::{
     as_handle::AsHandle,
-    bind::{CDataMut, HasDataType},
+    bind::{CDataMut, DelayedInput, HasDataType},
     buffer::{buf_ptr, clamp_small_int, mut_buf_ptr},
     column_description::{ColumnDescription, Nullability},
     data_type::DataType,
     drop_handle,
     error::{Error, IntoResult},
+    CData,
 };
 use odbc_sys::{
     Desc, FreeStmtOption, HDbc, HStmt, Handle, HandleType, Len, ParamType, Pointer, SQLBindCol,
     SQLBindParameter, SQLCloseCursor, SQLColAttributeW, SQLColumnsW, SQLDescribeColW,
     SQLDescribeParam, SQLExecDirectW, SQLExecute, SQLFetch, SQLFreeStmt, SQLGetData,
-    SQLNumResultCols, SQLParamData, SQLPrepareW, SQLSetStmtAttrW, SqlDataType, SqlReturn,
-    StatementAttribute, ULen,
+    SQLNumResultCols, SQLParamData, SQLPrepareW, SQLPutData, SQLSetStmtAttrW, SqlDataType,
+    SqlReturn, StatementAttribute, ULen,
 };
 use std::{convert::TryInto, ffi::c_void, marker::PhantomData, mem::ManuallyDrop, ptr::null_mut};
 use widestring::U16Str;
@@ -58,11 +59,6 @@ impl<'s> StatementImpl<'s> {
         // We do not want to run the drop handler, but transfer ownership instead.
         ManuallyDrop::new(self).handle
     }
-
-    /// Gain access to the underlying statement handle without transferring ownership to it.
-    pub fn as_sys(&self) -> HStmt {
-        self.handle
-    }
 }
 
 /// An ODBC statement handle. In this crate it is implemented by [`self::StatementImpl`]. In ODBC
@@ -74,6 +70,9 @@ impl<'s> StatementImpl<'s> {
 /// into account. It also allows for the trait to be implemented by a handle taking ownership of
 /// both, the statement and the connection.
 pub trait Statement {
+    /// Gain access to the underlying statement handle without transferring ownership to it.
+    fn as_sys(&self) -> HStmt;
+
     /// Binds application data buffers to columns in the result set.
     ///
     /// * `column_number`: `0` is the bookmark column. It is not included in some result sets. All
@@ -230,7 +229,7 @@ pub trait Statement {
     unsafe fn bind_input_parameter(
         &mut self,
         parameter_number: u16,
-        parameter: &(impl HasDataType + ?Sized),
+        parameter: &(impl HasDataType + CData + ?Sized),
     ) -> Result<(), Error>;
 
     /// Binds a buffer holding a single parameter to a parameter marker in an SQL statement. To bind
@@ -249,6 +248,22 @@ pub trait Statement {
         parameter: &mut (impl CDataMut + HasDataType),
     ) -> Result<(), Error>;
 
+    /// Binds an input stream to a parameter marker in an SQL statement. Use this to stream large
+    /// values at statement execution time. To bind preallocated constant buffers see
+    /// [`Statement::bind_input_parameter`].
+    ///
+    /// See <https://docs.microsoft.com/en-us/sql/odbc/reference/syntax/sqlbindparameter-function>.
+    ///
+    /// # Safety
+    ///
+    /// * It is up to the caller to ensure the lifetimes of the bound parameters.
+    /// * Calling this function may influence other statements that share the APD.
+    unsafe fn bind_delayed_input_parameter(
+        &mut self,
+        parameter_number: u16,
+        parameter: &mut (impl DelayedInput + HasDataType),
+    ) -> Result<(), Error>;
+
     /// `true` if a given column in a result set is unsigned or not a numeric type, `false`
     /// otherwise.
     ///
@@ -333,9 +348,20 @@ pub trait Statement {
         table_name: &U16Str,
         column_name: &U16Str,
     ) -> Result<(), Error>;
+
+    /// To put a batch of binary data into the data source at statement execution time. Returns true
+    /// if the `NEED_DATA` is returned by the driver.
+    ///
+    /// Panics if batch is empty.
+    fn put_binary_batch(&mut self, batch: &[u8]) -> Result<bool, Error>;
 }
 
 impl<'o> Statement for StatementImpl<'o> {
+    /// Gain access to the underlying statement handle without transferring ownership to it.
+    fn as_sys(&self) -> HStmt {
+        self.handle
+    }
+
     unsafe fn bind_col(
         &mut self,
         column_number: u16,
@@ -426,14 +452,18 @@ impl<'o> Statement for StatementImpl<'o> {
         }
     }
 
+    /// # Return
+    ///
+    /// Returns `true`, if SQLExecuteDirect return NEED_DATA indicating the need to send data at
+    /// execution.
     unsafe fn exec_direct(&mut self, statement_text: &U16Str) -> Result<bool, Error> {
         match SQLExecDirectW(
             self.handle,
             buf_ptr(statement_text.as_slice()),
             statement_text.len().try_into().unwrap(),
         ) {
-            SqlReturn::NO_DATA => Ok(false),
-            other => other.into_result(self).map(|()| true),
+            SqlReturn::NEED_DATA => Ok(true),
+            other => other.into_result(self).map(|()| false),
         }
     }
 
@@ -465,12 +495,12 @@ impl<'o> Statement for StatementImpl<'o> {
     ///
     /// # Return
     ///
-    /// If an update, insert, or delete statement did not affect any rows at the data source `false`
-    /// is returned.
+    /// Returns `true`, if SQLExecute return NEED_DATA indicating the need to send data at
+    /// execution.
     unsafe fn execute(&mut self) -> Result<bool, Error> {
         match SQLExecute(self.handle) {
-            SqlReturn::NO_DATA => Ok(false),
-            other => other.into_result(self).map(|()| true),
+            SqlReturn::NEED_DATA => Ok(true),
+            other => other.into_result(self).map(|()| false),
         }
     }
 
@@ -552,7 +582,7 @@ impl<'o> Statement for StatementImpl<'o> {
     unsafe fn bind_input_parameter(
         &mut self,
         parameter_number: u16,
-        parameter: &(impl HasDataType + ?Sized),
+        parameter: &(impl HasDataType + CData + ?Sized),
     ) -> Result<(), Error> {
         let parameter_type = parameter.data_type();
         SQLBindParameter(
@@ -603,6 +633,38 @@ impl<'o> Statement for StatementImpl<'o> {
         .into_result(self)
     }
 
+    /// Binds an input stream to a parameter marker in an SQL statement. Use this to stream large
+    /// values at statement execution time. To bind preallocated constant buffers see
+    /// [`Statement::bind_input_parameter`].
+    ///
+    /// See <https://docs.microsoft.com/en-us/sql/odbc/reference/syntax/sqlbindparameter-function>.
+    ///
+    /// # Safety
+    ///
+    /// * It is up to the caller to ensure the lifetimes of the bound parameters.
+    /// * Calling this function may influence other statements that share the APD.
+    unsafe fn bind_delayed_input_parameter(
+        &mut self,
+        parameter_number: u16,
+        parameter: &mut (impl DelayedInput + HasDataType),
+    ) -> Result<(), Error> {
+        let paramater_type = parameter.data_type();
+        SQLBindParameter(
+            self.handle,
+            parameter_number,
+            ParamType::Input,
+            parameter.cdata_type(),
+            paramater_type.data_type(),
+            paramater_type.column_size(),
+            paramater_type.decimal_digits(),
+            parameter.stream_ptr(),
+            0,
+            // We cast const to mut here, but we specify the input_output_type as input.
+            parameter.indicator_ptr() as *mut isize,
+        )
+        .into_result(self)
+    }
+
     /// `true` if a given column in a result set is unsigned or not a numeric type, `false`
     /// otherwise.
     ///
@@ -643,6 +705,9 @@ impl<'o> Statement for StatementImpl<'o> {
             SqlDataType::EXT_VAR_BINARY => DataType::Varbinary {
                 length: self.col_octet_length(column_number)?.try_into().unwrap(),
             },
+            SqlDataType::EXT_LONG_VAR_BINARY => DataType::LongVarbinary {
+                length: self.col_octet_length(column_number)?.try_into().unwrap(),
+            },
             SqlDataType::EXT_BINARY => DataType::Binary {
                 length: self.col_octet_length(column_number)?.try_into().unwrap(),
             },
@@ -652,6 +717,9 @@ impl<'o> Statement for StatementImpl<'o> {
             SqlDataType::EXT_W_CHAR => DataType::WChar {
                 length: self.col_display_size(column_number)?.try_into().unwrap(),
             },
+            SqlDataType::EXT_LONG_VARCHAR => DataType::LongVarchar {
+                length: self.col_display_size(column_number)?.try_into().unwrap(),
+            },
             SqlDataType::CHAR => DataType::Char {
                 length: self.col_display_size(column_number)?.try_into().unwrap(),
             },
@@ -875,6 +943,29 @@ impl<'o> Statement for StatementImpl<'o> {
             .into_result(self)
         }
     }
+
+    /// To put a batch of binary data into the data source at statement execution time. Returns true
+    /// if the `NEED_DATA` is returned by the driver.
+    ///
+    /// Panics if batch is empty.
+    fn put_binary_batch(&mut self, batch: &[u8]) -> Result<bool, Error> {
+        // Probably not strictly necessary. MSSQL returns an error than inserting empty batches.
+        // Still strikes me as a programming error. Maybe we could also do nothing instead.
+        if batch.is_empty() {
+            panic!("Attempt to put empty batch into data source.")
+        }
+
+        unsafe {
+            match SQLPutData(
+                self.handle,
+                batch.as_ptr() as Pointer,
+                batch.len().try_into().unwrap(),
+            ) {
+                SqlReturn::NEED_DATA => Ok(true),
+                other => other.into_result(self).map(|()| false),
+            }
+        }
+    }
 }
 
 /// Description of a parameter associated with a parameter marker in a prepared statement. Returned
