@@ -6,8 +6,9 @@
 //! * `&a` -> Single input parameter
 //! * `&mut a` -> Input Output parameter
 //! * `Out(&mut a)` -> Output parameter
-//! * `(&a,&b,&c)` -> Fixed number of parameters
+//! * `(a,b,c)` -> Fixed number of parameters
 //! * `&[a]` -> Arbitrary number of parameters
+//! * `&mut BlobParam` -> Stream long input parameters.
 //! * `Box<dyn InputParameter>` -> Aribtrary input parameter
 //! * `&[Box<dyn InputParameter>]` -> Aribtrary number of arbitrary input parameters
 //! * `a.into_parameter()` -> Convert idiomatic Rust type into something bindable by ODBC.
@@ -60,10 +61,10 @@
 //! # Ok::<(), odbc_api::Error>(())
 //! ```
 //!
-//! In that case it is likely that the driver manager converts our annotated year into a string which
-//! is most likely being converted back into an integer by the driver. All this converting can be
-//! confusing, but it is helpful if we do not know what types the parameters actually have (i.e. the
-//! query could have been entered by the user on the command line.). There is also an option to
+//! In that case it is likely that the driver manager converts our annotated year into a string
+//! which is most likely being converted back into an integer by the driver. All this converting can
+//! be confusing, but it is helpful if we do not know what types the parameters actually have (i.e.
+//! the query could have been entered by the user on the command line.). There is also an option to
 //! query the parameter types beforehand, but my advice is not trust the information blindly if you
 //! cannot test this with your driver beforehand.
 //!
@@ -181,6 +182,109 @@
 //! # Ok::<(), odbc_api::Error>(())
 //! ```
 //!
+//! ## Sending long data
+//!
+//! Many ODBC drivers have size limits of how big parameters can be. Apart from that you may not
+//! want to allocate really large buffers in your application in order to keep a small memory
+//! footprint. Luckily ODBC also supports streaming data to the database batch by batch at statement
+//! execution time. To support this, this crate offers the [`BlobParam`], which can be bound as a
+//! mutable reference. An instance of [`BlobParam`] is usually created by calling
+//! [`Blob::as_blob_param`] from a wrapper implenting [`Blob`].
+//!
+//! ### Inserting long binary data from a file.
+//!
+//! [`BlobRead::from_path`] is the most convinient way to turn a file path into a [`Blob`]
+//! parameter. The following example also demonstrates that the streamed blob parameter can be
+//! combined with reqular input parmeters like `id`.
+//!
+//! ```
+//! use std::{error::Error, path::Path};
+//! use odbc_api::{Connection, parameter::{Blob, BlobRead}, IntoParameter};
+//!
+//! fn insert_image_to_db(
+//!     conn: &Connection<'_>,
+//!     id: &str,
+//!     image_path: &Path) -> Result<(), Box<dyn Error>>
+//! {
+//!     let mut blob = BlobRead::from_path(&image_path)?;
+//!
+//!     let sql = "INSERT INTO Images (id, image_data) VALUES (?, ?)";
+//!     let parameters = (&id.into_parameter(), &mut blob.as_blob_param());
+//!     conn.execute(sql, parameters)?;
+//!     Ok(())
+//! }
+//! ```
+//!
+//! ### Inserting long binary data from any `io::BufRead`.
+//!
+//! This is more flexible than inserting just from files. Note however that files provide metadata
+//! about the length of the data, which `io::BufRead` does not. This is not an issue for most
+//! drivers, but some can perform optimization if they know the size in advance. In the tests
+//! SQLite has shown a bug to only insert empty data if no size hint has been provided.
+//!
+//! ```
+//! use std::io::BufRead;
+//! use odbc_api::{Connection, parameter::{Blob, BlobRead}, IntoParameter, Error};
+//!
+//! fn insert_image_to_db(
+//!     conn: &Connection<'_>,
+//!     id: &str,
+//!     image_data: impl BufRead) -> Result<(), Error>
+//! {
+//!     const MAX_IMAGE_SIZE: usize = 4 * 1024 * 1024;
+//!     let mut blob = BlobRead::with_upper_bound(image_data, MAX_IMAGE_SIZE);
+//!
+//!     let sql = "INSERT INTO Images (id, image_data) VALUES (?, ?)";
+//!     let parameters = (&id.into_parameter(), &mut blob.as_blob_param());
+//!     conn.execute(sql, parameters)?;
+//!     Ok(())
+//! }
+//! ```
+//!
+//! ### Inserting long strings
+//!
+//! This example insert `title` as a normal input parameter but streams the potentially much longer
+//! `String` in `text` to the database as a large text blob. This allows to circumvent the size
+//! restrictions for `String` arguments of many drivers (usually around 4 or 8 KiB).
+//!
+//! ```
+//! use odbc_api::{Connection, parameter::{Blob, BlobSlice}, IntoParameter, Error};
+//!
+//! fn insert_book(
+//!     conn: &Connection<'_>,
+//!     title: &str,
+//!     text: &str
+//! ) -> Result<(), Error>
+//! {
+//!     let mut blob = BlobSlice::from_text(text);
+//! 
+//!     let insert = "INSERT INTO Books (title, text) VALUES (?,?)";
+//!     let parameters = (title.into_parameter(), &mut blob.as_blob_param());
+//!     conn.execute(&insert, &mut blob.as_blob_param())?;
+//!     Ok(())
+//! }
+//! ```
+//!
+//! ### Inserting long binary data from `&[u8]`.
+//!
+//! ```
+//! use odbc_api::{Connection, parameter::{Blob, BlobSlice}, IntoParameter, Error};
+//!
+//! fn insert_image(
+//!     conn: &Connection<'_>,
+//!     id: &str,
+//!     image_data: &[u8]
+//! ) -> Result<(), Error>
+//! {
+//!     let mut blob = BlobSlice::from_byte_slice(image_data);
+//! 
+//!     let insert = "INSERT INTO Images (id, image_data) VALUES (?,?)";
+//!     let parameters = (&id.into_parameter(), &mut blob.as_blob_param());
+//!     conn.execute(&insert, &mut blob.as_blob_param())?;
+//!     Ok(())
+//! }
+//! ```
+//!
 //! ## Passing the type you absolutely think should work, but does not.
 //!
 //! Sadly not every type can be safely bound as something the ODBC C-API understands. Most prominent
@@ -198,8 +302,8 @@
 //! # Ok::<(), odbc_api::Error>(())
 //! ```
 //!
-//! Alas, not all is lost. We can still make use of the [`crate::IntoParameter`] trait to convert it into
-//! something that works.
+//! Alas, not all is lost. We can still make use of the [`crate::IntoParameter`] trait to convert it
+//! into something that works.
 //!
 //! ```no_run
 //! use odbc_api::{Environment, IntoParameter};
@@ -223,11 +327,13 @@
 //! work? Well, in that case please open an issue or a pull request. [`crate::IntoParameter`] can usually be
 //! implemented entirely in safe code, and is a suitable spot to enable support for your custom
 //! types.
+mod blob;
 mod c_string;
 mod varbin;
 mod varchar;
 
 pub use self::{
+    blob::{Blob, BlobParam, BlobRead, BlobSlice},
     varbin::{VarBinary, VarBinaryArray, VarBinaryBox, VarBinarySlice, VarBinarySliceMut},
     varchar::{VarChar, VarCharArray, VarCharBox, VarCharSlice, VarCharSliceMut},
 };
@@ -247,14 +353,14 @@ use crate::{
 ///
 /// Considerations for implementers
 ///
-/// Extend the [`crate::handles::HasDataType`] trait with the guarantee, that the bound parameter buffer
-/// contains at least one element.
+/// Extend the [`crate::handles::HasDataType`] trait with the guarantee, that the bound parameter
+/// buffer contains at least one element.
 ///
 /// Since the indicator provided by implementation is used to indicate the length of the value in
 /// the buffer, care must be taken to prevent out of bounds access in case the implementation also
-/// is used as an output parameter, an contains truncated values (i.e. the indicator is longer than
+/// is used as an output parameter, and contains truncated values (i.e. the indicator is longer than
 /// the buffer and the value within).
-pub unsafe trait InputParameter: HasDataType {}
+pub unsafe trait InputParameter: HasDataType + CData {}
 
 /// Guarantees that there is space in the output buffer for at least one element.
 pub unsafe trait Output: CDataMut + HasDataType {}
