@@ -3,7 +3,7 @@ use odbc_api::{
     buffers,
     buffers::TextColumn,
     handles::{CDataMut, Statement},
-    Connection, Cursor, Environment, RowSetBuffer, U16Str,
+    Connection, Cursor, Environment, Error, RowSetBuffer, U16Str,
 };
 
 // Rust by default executes tests in parallel. Yet only one environment is allowed at a time.
@@ -24,6 +24,13 @@ pub struct Profile {
     pub blob_type: &'static str,
 }
 
+impl Profile {
+    /// Open a new connection using the connection string of the profile
+    pub fn connection(&self) -> Result<Connection<'static>, Error> {
+        ENV.connect_with_connection_string(self.connection_string)
+    }
+}
+
 /// Creates the table and assures it is empty. Columns are named a,b,c, etc.
 pub fn setup_empty_table(
     conn: &Connection<'_>,
@@ -33,7 +40,7 @@ pub fn setup_empty_table(
 ) -> Result<(), odbc_api::Error> {
     let drop_table = &format!("DROP TABLE IF EXISTS {}", table_name);
 
-    let column_names = &["a", "b", "c", "d", "e", "f", "g", "h"];
+    let column_names = &["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"];
     let cols = column_types
         .iter()
         .zip(column_names)
@@ -65,11 +72,11 @@ pub fn cursor_to_string(cursor: impl Cursor) -> String {
     while let Some(row_set) = row_set_cursor.fetch().unwrap() {
         for row_index in 0..row_set.num_rows() {
             if row_index != 0 {
-                text.push_str("\n");
+                text.push('\n');
             }
             for col_index in 0..row_set.num_cols() {
                 if col_index != 0 {
-                    text.push_str(",");
+                    text.push(',');
                 }
                 text.push_str(
                     row_set
